//-----------------------------------------------------------------------
// File : AsdxShaderUtil.h
// Desc : Asura DirectX Shader Utility
// Copyright(c) Project Asura. All right resreved.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// History
//-----------------------------------------------------------------------
// <---> 2011/07/23 [Pocol] VK쐬
// <End>

#ifndef __ASURA_DIRECTX_SHADER_UTIL_H__
#define __ASURA_DIRECTX_SHADER_UTIL_H__

//-----------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------
#include "asDef.h"
#include <d3d11.h>
#include <vector>

namespace Asura {
namespace Graphics {

/////////////////////////////////////////////////////////////////////////
// ShaderUtil class
/////////////////////////////////////////////////////////////////////////
class ShaderUtil
{
    //-------------------------------------------------------------------
    // list of friend classes and methods
    //-------------------------------------------------------------------
    /* NOTHING */

private:
    //-------------------------------------------------------------------
    // private variables
    //-------------------------------------------------------------------
    /* NOTHING */

    //-------------------------------------------------------------------
    // private methods
    //-------------------------------------------------------------------
    /* NOTHING */

protected:
    //-------------------------------------------------------------------
    // protected variables
    //-------------------------------------------------------------------
    /* NOTHING */

    //-------------------------------------------------------------------
    // protected methods
    //-------------------------------------------------------------------
    /* NOTHING */

public:
    //-------------------------------------------------------------------
    // public variables
    //-------------------------------------------------------------------
    /* NOTHING */

    //-------------------------------------------------------------------
    // public methods
    //-------------------------------------------------------------------

    // Compile From File
	static bool CompileFromFileA( 
		const char* filename, 
		const char* entryPoint,
		const char* shaderModel,
		ID3DBlob** ppBlobOut );
	static bool CompileFromFileW(
		const wchar_t* filename,
		const char* entryPoint,
		const char* shaderModel,
		ID3DBlob** ppBlobOut );
	static bool CompileFromFile(
		const tchar* filename,
		const char* entryPoint,
		const char* shaderModel,
		ID3DBlob** ppBlobOut );

    // Create Vertex Shader From File
    static bool CreateVertexShaderFromFileA(
        const char* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3DBlob** ppBlob,
        ID3D11VertexShader** ppShader );
    static bool CreateVertexShaderFromFileW(
        const wchar_t* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3DBlob** ppBlob,
        ID3D11VertexShader** ppShader );
    static bool CreateVertexShaderFromFile(
        const tchar* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3DBlob** ppBlob,
        ID3D11VertexShader** ppShader );

    // Create Pixel Shader From File
    static bool CreatePixelShaderFromFileA(
        const char* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3D11PixelShader** ppShader );
    static bool CreatePixelShaderFromFileW(
        const wchar_t* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3D11PixelShader** ppShader );
    static bool CreatePixelShaderFromFile(
        const tchar* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3D11PixelShader** ppShader );

    // Create Geometry Shader From File
    static bool CreateGeometryShaderFromFileA(
        const char* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3D11GeometryShader** ppShader );
    static bool CreateGeometryShaderFromFileW(
        const wchar_t* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3D11GeometryShader** ppShader );
   static bool CreateGeometryShaderFromFile(
        const tchar* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3D11GeometryShader** ppShader );

   // Create Hull Shader From File
   static bool CreateHullShaderFromFileA(
        const char* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3D11HullShader** ppShader );
   static bool CreateHullShaderFromFileW(
        const wchar_t* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3D11HullShader** ppShader );
   static bool CreateHullShaderFromFile(
        const tchar* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3D11HullShader** ppShader );

   // Create Domain Shader From File
   static bool CreateDomainShaderFromFileA(
        const char* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3D11DomainShader** ppShader );
   static bool CreateDomainShaderFromFileW(
        const wchar_t* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3D11DomainShader** ppShader );
   static bool CreateDomainShaderFromFile(
        const tchar* filename,
        const char* entryPoint,
        const char* shaderModel,
        ID3D11Device* pDevice,
        ID3D11DomainShader** ppShader );

   // Create Compute Shader From File
   static bool CreateComputeShaderFromFileA(
      const char* filename,
      const char* entryPoint,
      const char* shaderModel,
      ID3D11Device* pDevice,
      ID3D11ComputeShader** ppShader );
   static bool CreateComputeShaderFromFileW(
      const wchar_t* filename,
      const char* entryPoint,
      const char* shaderModel,
      ID3D11Device* pDevice,
      ID3D11ComputeShader** ppShader );
   static bool CreateComputeShaderFromFile(
      const tchar* filename,
      const char* entryPoint,
      const char* shaderModel,
      ID3D11Device* pDevice,
      ID3D11ComputeShader** ppShader );
};


//////////////////////////////////////////////////////////////////////////
// SHADER_TYPE enum
//////////////////////////////////////////////////////////////////////////
enum SHADER_TYPE
{
	SHADER_TYPE_VERTEX = 0,
	SHADER_TYPE_PIXEL,
	SHADER_TYPE_GEOMETRY,
	SHADER_TYPE_HULL,
	SHADER_TYPE_DOMAIN,
	SHADER_TYPE_COMPUTE,

	NUM_SHADER_TYPE
};


/////////////////////////////////////////////////////////////////////////
// ShaderCompileDesc structure
/////////////////////////////////////////////////////////////////////////
struct ShaderCompileDesc
{
	uint32_t	shaderType;		//!< VF[_^Cv
	const char*	pEntryPoint;	//!< Gg[|Cg
	const char*	pShaderModel;	//!< VF[_f
};


/////////////////////////////////////////////////////////////////////////
// ShaderContainer structure
/////////////////////////////////////////////////////////////////////////
class ShaderContainer
{
	//------------------------------------------------------------------
	// list of friend classes and methods
	//------------------------------------------------------------------
	/* NOTHING */

private:
	//------------------------------------------------------------------
	// private variables
	//------------------------------------------------------------------
	std::vector< ID3DBlob* >				mVSBlob;
	std::vector< ID3D11VertexShader* >		mVS;
	std::vector< ID3D11PixelShader* >		mPS;
	std::vector< ID3D11GeometryShader* >	mGS;
	std::vector< ID3D11HullShader* >		mHS;
	std::vector< ID3D11DomainShader* >		mDS;
	std::vector< ID3D11ComputeShader* >		mCS;

	//------------------------------------------------------------------
	// private methods
	//------------------------------------------------------------------
	/* NOTHING */

protected:
	//------------------------------------------------------------------
	// proected variables
	//------------------------------------------------------------------
	/* NOTHING */

	//------------------------------------------------------------------
	// protected methods
	//------------------------------------------------------------------
	/* NOTHING */

public:
	//------------------------------------------------------------------
	// public variables
	//------------------------------------------------------------------
	/* NOTHING */

	//------------------------------------------------------------------
	// public methods
	//------------------------------------------------------------------
	ShaderContainer();
	~ShaderContainer();
	void Release();
	bool CreateFromFileW( 
		ID3D11Device* pDevice,
		wchar_t* pFilename, 
		ShaderCompileDesc *pDescs,
		const int numDescs );
	bool CreateFromFileA( 
		ID3D11Device* pDevice,
		char* pFilename, 
		ShaderCompileDesc *pDescs,
		const int numDescs );
	bool CreateFromFile(
		ID3D11Device* pDevice,
		tchar* pFilename,
		ShaderCompileDesc* pDescs,
		const int numDescs );
	bool CreateInputLayout( 
		const uint32_t index,
		const D3D11_INPUT_ELEMENT_DESC* pInputElementDesc,
		const uint32_t numElements, 
		ID3D11InputLayout* ppInputLayout );
	void SetShader( 
		ID3D11DeviceContext* pImmediateContext,
		int indexVS,
		int indexPS = -1,
		int indexGS = -1,
		int indexHS = -1,
		int indexDS = -1 );
    void SetComputeShader(
        ID3D11DeviceContext* pImmediateContext,
        int indexCS );
	uint32_t GetNumVS() { return mVS.size(); }
	uint32_t GetNumPS() { return mPS.size(); }
	uint32_t GetNumGS() { return mGS.size(); }
	uint32_t GetNumHS() { return mHS.size(); }
	uint32_t GetNumDS() { return mDS.size(); }
	uint32_t GetNumCS() { return mCS.size(); }
	ID3D11VertexShader*		GetVS( const uint32_t index ) { return mVS[ index ]; }
	ID3D11PixelShader*		GetPS( const uint32_t index ) { return mPS[ index ]; }
	ID3D11GeometryShader*	GetGS( const uint32_t index ) { return mGS[ index ]; }
	ID3D11HullShader*		GetHS( const uint32_t index ) { return mHS[ index ]; }
	ID3D11DomainShader*		GetDS( const uint32_t index ) { return mDS[ index ]; }
	ID3D11ComputeShader*	GetCS( const uint32_t index ) { return mCS[ index ]; }

};


} // namespace Graphics
} // namespace Asura

#endif//__ASURA_DIRECTX_SHADER_UTIL_H__