//--------------------------------------------------------------------
// File : Lambert.hlsl
// Desc : Lambert Model
// Copyright(c) Project Asura. All right reserved.
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Constant Buffer and Global Variables
//--------------------------------------------------------------------

//!< sobt@
cbuffer VSMatrix : register( b0 )
{
    float4x4 World : packoffset( c0 );		//!< [hs
    float4x4 View  : packoffset( c4 );		//!< r[s
    float4x4 Proj  : packoffset( c8 );		//!< ˉes
};

//!< }eAobt@
cbuffer PSMaterial : register( b1 )
{
    float3 Diffuse : packoffset( c0 );		//!< gUːF
    float  Alpha   : packoffset( c0.w );	//!< sx
    float3 Specular: packoffset( c1 );		//!< ʔːF
    float  Power   : packoffset( c1.w );	//!< ʔˋx
    float3 Emissive: packoffset( c2 );		//!< ȏƖF
};

//!< xNgobt@
cbuffer PSVector : register( b2 )
{
    float3 LightDir     : packoffset( c0 );	//!< CgxNg
    float3 ViewDir      : packoffset( c1 );	//!< xNg
};

//!< CgJ[obt@
cbuffer PSLightColor : register( b3 )
{
    float3 LightAmbient : packoffset( c0 );	//!< Cg̊
    float3 LightDiffuse : packoffset( c1 );	//!< Cg̊gUˌ
};


///////////////////////////////////////////////////////////////////////
// VSInput structure
///////////////////////////////////////////////////////////////////////
struct VSInput
{
    float3 Position : POSITION;		//!< ʒuW
    float3 Normal   : NORMAL;		//!< @xNg
    float2 TexCoord : TEXCOORD;		//!< eNX`W
};


///////////////////////////////////////////////////////////////////////
// VSOutput structure
///////////////////////////////////////////////////////////////////////
struct VSOutput
{
    float4 Position : SV_POSITION;	//!< ʒuW
    float2 TexCoord : TEXCOORD0;	//!< eNX`W
    float3 Normal   : TEXCOORD1;	//!< @xNg
};


//---------------------------------------------------------------------
//! @brief _VF[_CGg[|Cg
//! @param [in] input _̓f[^
//! @return _o̓f[^
//---------------------------------------------------------------------
VSOutput VSFunc( VSInput input )
{
    VSOutput output = (VSOutput)0;

    float4 localPos = float4( input.Position, 1.0f );
    float4 worldPos = mul( World, localPos );
    float4 viewPos  = mul( View, worldPos );
    float4 projPos  = mul( Proj, viewPos );

    output.Position = projPos;
    output.TexCoord = input.TexCoord;
    output.Normal   = input.Normal;

    return output;
}


//---------------------------------------------------------------------
//! @brief sNZVF[_CGg[|Cg
//! @param [in] input _o̓f[^
//! @return sNZo̓f[^
//---------------------------------------------------------------------
float4 PSFunc( VSOutput input ) : SV_TARGET
{
    float4 output = (float4)0;

    // K
    float3 L = normalize( LightDir );
    float3 N = normalize( input.Normal );

    // F
    float3 ambient = LightAmbient;

    // gUːF
    float3 diffuse = LightDiffuse * Diffuse * max( dot( N, L ), 0.0f );

    // o͐F
    output = float4( ambient + diffuse, Alpha );
    
    return output;
}